<?php	
	
	if(!defined('SYS_ROOT')){exit('No direct file access allowed!');}
	
	/**
	* Xtron PHP Framework
	* A MVC style framework for building web applications
	* By: Jason Alan Delaney (http://www.xtroncms.com)
	* PHP version 5+
	*/	
	
	/**
	* Template class for using custom tags in view files
	* and the use of global themes.
	*/
	
	class Template {
		
		var $config;//Config array from app/config/
		var $tags = array();
		var $tag_files;
		var $template;//View file name
		var $theme_dir = '';//Path to theme
		var $theme_name= '';
		var $cache;
		var $tag;//Tag name for content when using theme
		var $ext = '.php';//Template file extension (not really used in current ver. 1.0.0)
		var $data;//Variables to pass through to view file
		
		function __construct($tags = array()){
			$this->config =& load_class('Config');
			//Load template config file
			$this->config->load('template');
			
			if(count($tags) > 0 && is_array($tags)){
				foreach($tags as $tag => $val){
					$this->tags[$tag] = $val;
				}
			}
			
		}
		
		/**
		* Echo view file to the browser.
		* Cache functions
		* Themes
		*/
		function parse($template,$data=null, $cache=false){
			
			/**
			* TODO:
			* - Cache expiration
			* - Cache exists OR template exists
			* - Don't use replace_tags() if cached version is available
			*/
			
			$this->template = $template;
			$this->cache	= $cache;
			$this->data		= $data;
			
			$this->load_tags();
			if($this->template_exists($template) || $cache === true){
				/**
				* Render content through theme file if using one
				*/
				if($this->theme_dir != '' && !empty($this->theme_dir)){
					//Put page contents into tag
					$this->tags[$this->tag] = $this->replace_tags();
					//Set template to theme template
					$this->template = $this->theme_dir;
					
					echo $this->replace_tags($cache);
					
				}else{
					
					echo $this->replace_tags($cache);
				
				}
				return;
			}
			
			return false;
			
		} 
		
		/**
		* Set a theme file to use. If set, all view file data is passed into to theme file.
		*/
		function use_theme($theme, $ext = '.php'){
			
			/**
			* TODO: 
			* - Theme caching
			* - Custom theme extensions '.theme', '.template','.tpl' etc
			*/
			
			$theme = strtolower($theme);
			
			if(empty($theme) || $theme == ''){	
				error('Theme name can not be empty - function::use_theme()'. 'Template Error');
				return;
			}
			
			//Check if theme path is defined
			if(file_exists(APP_ROOT . 'views' . DS . $theme) && strpos($theme, $ext)){
			
				$this->theme_dir = APP_ROOT . 'views' . DS . $theme;
				$theme = explode('/',$theme);
				
				$this->theme_name = end($theme);
				
			}elseif(file_exists(APP_ROOT . 'views' . DS . $theme  . $ext)){
			
				$this->theme_dir = APP_ROOT . 'views' . DS . $theme  . $ext;
				$this->theme_name = $theme;
				
			}else{
				
				error('Theme file doesn\'t exist - function::use_theme()', 'Template Error');
				return;
				
			}
			
			$this->tag = str_replace(array('{','}'),'',$this->config->item('content_tag'));
			
		}
		
		
		/**
		* Load template tag files
		*/
		function load_tags(){
			
			if(isset($this->config->config['tag_files']) && is_array($this->config->config['tag_files']) && count($this->config->config['tag_files']) > 0){
				foreach($this->config->config['tag_files'] as $file){
					if(file_exists(APP_ROOT . 'tags' . DS . 'tags.' . $file . '.php')){
						include(APP_ROOT . 'tags' . DS . 'tags.' . $file . '.php');
					}
				}
				
				if(isset($tags)){
					$this->tags = $tags;
					return;
				}
				//In case tag file didn't actually have a $tag array pff
				$this->tags = array();
				
			}else{
				//No tag file specified or found?
				$this->tags = array();
				return;
			}
			
		}
		
		/**
		* Checks if a given template exists
		*/
		function template_exists($tpl, $ext = '.php'){
			$tpl = basename($tpl);
			
			$ext = (empty($ext))?  $this->ext : $ext;
			
			if(file_exists(APP_ROOT . 'views' . DS . $tpl . $ext)
			|| file_exists(APP_ROOT . 'views' . DS . $templ . $ext)
			){
				return true;
			}
			
			return false;
			
		}
		
		/**
		* @bool cache parsed view file? (TRUE|FALSE)
		* Replace custom tags
		*/
		function replace_tags(){
			//Get instance of the main controller so we can load the view file to parse
			$xpf =& controller_instance();
			
			//Get raw view file from the Display Object
			$view = $xpf->display->view($this->template,$this->data, $this->cache);

			//Replace tags if caching is off
			if($this->cache == false){
				echo $this->template;
				//Replace data in {tag} with data from tag file
				foreach ($this->tags as $tag => $data){
					$view = preg_replace("~{+" . $tag . "+}~", $data,$view);
				}
			}
			
			/**
			* Save cached version of file? 
			*/
			if($this->cache === true &&  $xpf->display->cache_exists($this->template) == false){

				$cname 	= SYS_ROOT . 'cache' . DS . 'cache.' . basename($this->template, '.php') . '.php';

				//Save the cached file 
				if(substr(basename($this->template),0, 1) !== '_'){
					//open/create cache file for writing
					$fh		= fopen($cname,'w+'); 
					fwrite($fh, $view); 
					fclose($fh);	
				}
			}
			
			return $view;
		}
		
	}
	
?>